@@ -33,8 +33,12 @@ from linshare_cli.common import VTable
 from linshare_cli.common import HTable
 from linshare_cli.core import LinShareException
 import argtoolbox
+import argparse
+import re
 
 
+# pylint: disable-msg=C0111
+# pylint: disable-msg=R0903
 # -----------------------------------------------------------------------------
 class DefaultCommand(common.DefaultCommand):
     """ Default command object use by the serer API. If you want to add a new
@@ -42,8 +46,11 @@ class DefaultCommand(common.DefaultCommand):
     """
 
     def __get_cli_object(self, args):
-        return AdminCli(args.host, args.user, args.password, args.verbose,
-                        args.debug, args.realm, args.application_name)
+        cli = AdminCli(args.host, args.user, args.password, args.verbose,
+                       args.debug)
+        if args.base_url:
+            cli.base_url = args.base_url
+        return cli
 
     def _run(self, method, message_ok, err_suffix, *args):
         try:
@@ -110,7 +117,7 @@ class DomainsListCommand(DefaultCommand):
         keys = self.ls.domains.get_rbu().get_keys(args.extended)
         table = None
         if args.vertical:
-            table = VTable(keys)
+            table = VTable(keys, debug=self.debug)
         else:
             table = HTable(keys)
             # styles
@@ -375,7 +382,7 @@ class LdapConnectionsListCommand(DefaultCommand):
         keys = self.ls.ldap_connections.get_rbu().get_keys()
         table = None
         if args.vertical:
-            table = VTable(keys)
+            table = VTable(keys, debug=self.debug)
         else:
             table = HTable(keys)
             # styles
@@ -475,7 +482,7 @@ class DomainPatternsListCommand(DefaultCommand):
         keys = self.ls.domain_patterns.get_rbu().get_keys(args.extended)
         table = None
         if args.vertical:
-            table = VTable(keys)
+            table = VTable(keys, debug=self.debug)
         else:
             table = HTable(keys)
             # styles
@@ -590,7 +597,7 @@ class ThreadsListCommand(DefaultCommand):
         keys = self.ls.threads.get_rbu().get_keys(args.extended)
         table = None
         if args.vertical:
-            table = VTable(keys)
+            table = VTable(keys, debug=self.debug)
         else:
             table = HTable(keys)
             # styles
@@ -618,7 +625,7 @@ class ThreadMembersListCommand(DefaultCommand):
         keys = self.ls.thread_members.get_rbu().get_keys(args.extended)
         table = None
         if args.vertical:
-            table = VTable(keys)
+            table = VTable(keys, debug=self.debug)
         else:
             table = HTable(keys)
             # styles
@@ -649,11 +656,15 @@ class UsersListCommand(DefaultCommand):
 
     def __call__(self, args):
         super(UsersListCommand, self).__call__(args)
-        json_obj = self.ls.users.list()
+        json_obj = self.ls.users.search(args.firstname, args.lastname,
+                                        args.mail)
+        self.format_date(json_obj, 'creationDate')
+        self.format_date(json_obj, 'modificationDate')
+        self.format_date(json_obj, 'expirationDate')
         keys = self.ls.users.get_rbu().get_keys(args.extended)
         table = None
         if args.vertical:
-            table = VTable(keys)
+            table = VTable(keys, debug=self.debug)
         else:
             table = HTable(keys)
             # styles
@@ -661,16 +672,276 @@ class UsersListCommand(DefaultCommand):
             table.padding_width = 1
         table.sortby = "mail"
         table.reversesort = args.reverse
+
         def filters(row):
             if not args.identifiers:
                 return True
-            if row.get('mail') in args.identifiers:
+            if re.search(r"^.*(" + "|".join(args.identifiers) + ").*$",
+                         row.get('uuid')):
                 return True
             return False
         table.print_table(json_obj, keys, filters)
         return True
 
 
+# -------------------------- Functionalities ------------------------------------------
+# -----------------------------------------------------------------------------
+class FunctionalityListCommand(DefaultCommand):
+    """ List all functionalities."""
+
+    def __call__(self, args):
+        super(FunctionalityListCommand, self).__call__(args)
+        json_obj = self.ls.funcs.list(args.domain)
+        keys = self.ls.funcs.get_rbu().get_keys(args.extended)
+        table = None
+        if args.vertical:
+            table = VTable(keys, debug=self.debug)
+        else:
+            table = HTable(keys)
+            # styles
+            table.align["identifier"] = "l"
+            table.padding_width = 1
+        table.sortby = "identifier"
+        table.reversesort = args.reverse
+
+        def filters(row):
+            if not args.identifiers:
+                return True
+            if re.search(r"^.*(" + "|".join(args.identifiers) + ").*$",
+                         row.get('identifier')):
+                return True
+            return False
+        table.print_table(json_obj, keys, filters)
+        return True
+
+    def complete(self, args, prefix):
+        super(FunctionalityListCommand, self).__call__(args)
+        json_obj = self.ls.funcs.list(args.domain)
+        return (v.get('identifier')
+                for v in json_obj if v.get('identifier').startswith(prefix))
+
+    def complete_domain(self, args, prefix):
+        super(FunctionalityListCommand, self).__call__(args)
+        json_obj = self.ls.domains.list()
+        return (v.get('identifier')
+                for v in json_obj if v.get('identifier').startswith(prefix))
+
+# -----------------------------------------------------------------------------
+class FunctionalityDisplayCommand(DefaultCommand):
+    """ List all functionalities."""
+
+    def __call__(self, args):
+        super(FunctionalityDisplayCommand, self).__call__(args)
+        table = HTable()
+        table.field_names = ["Name", "Values"]
+        # styles
+        table.align["Name"] = "l"
+        table.align["Values"] = "l"
+        table.padding_width = 1
+        json_obj = self.ls.funcs.get(args.identifier, args.domain)
+        if self.debug:
+            self.pretty_json(json_obj)
+        if not json_obj.get('displayable'):
+            print "You can not modify this functionality in this domain"
+            return True
+        table.add_row(['Identifier', json_obj.get('identifier')])
+        table.add_row(['Domain', json_obj.get('domain')])
+        apo = json_obj.get('activationPolicy')
+        if not apo.get('system') and apo.get('parentAllowUpdate'):
+            table.add_row(['Activation Policy',
+                           self.format_policy(apo)])
+        cpo = json_obj.get('configurationPolicy')
+        if not cpo.get('system') and cpo.get('parentAllowUpdate'):
+            table.add_row(['Configuration Policy',
+                           self.format_policy(cpo)])
+        dpo = json_obj.get('delegationPolicy')
+        if dpo is not None:
+            if not dpo.get('system') and dpo.get('parentAllowUpdate'):
+                table.add_row(['Delegation Policy',
+                               self.format_policy(dpo)])
+        param = self.format_parameters(json_obj)
+        if param:
+            cpt = 0
+            for param in param:
+                cpt += 1
+                if cpt > 1:
+                    name = u'Parameters' + str(cpt)
+                else:
+                    name = u'Parameters'
+                if json_obj.get('parentAllowParametersUpdate'):
+                    table.add_row([name, param])
+        out = table.get_string()
+        print unicode(out)
+        return True
+
+    def format_policy(self, row):
+        if row is None:
+            return row
+        d_format = "Status : {status!s:13s} | Policy : {policy:10s}"
+        return unicode(d_format).format(**row)
+
+    def format_parameters(self, json_obj):
+        ret = []
+        for param in json_obj.get('parameters'):
+            row = {}
+            f_type = param.get('type')
+            row['type'] = f_type
+            if  f_type == "INTEGER":
+                row['value'] = param.get('integer')
+            elif  f_type == "STRING":
+                row['value'] = param.get('string')
+            if  f_type == "UNIT_SIZE":
+                row['value'] = str(param.get('integer')) + " " + param.get('string')
+            elif  f_type == "UNIT_TIME":
+                row['value'] = str(param.get('integer')) + " " + param.get('string')
+            elif  f_type == "BOOLEAN":
+                row['value'] = param.get('bool')
+            if row:
+                d_format = "Type   : {type!s:13s} | Value  : {value!s}"
+                ret.append(unicode(d_format).format(**row))
+        return ret
+
+    def complete(self, args, prefix):
+        super(FunctionalityDisplayCommand, self).__call__(args)
+        json_obj = self.ls.funcs.list(args.domain)
+        return (v.get('identifier')
+                for v in json_obj if v.get('identifier').startswith(prefix))
+
+    def complete_domain(self, args, prefix):
+        super(FunctionalityDisplayCommand, self).__call__(args)
+        json_obj = self.ls.domains.list()
+        return (v.get('identifier')
+                for v in json_obj if v.get('identifier').startswith(prefix))
+
+
+# -----------------------------------------------------------------------------
+class FunctionalityUpdateCommand(DefaultCommand):
+    """ List all functionalities."""
+
+    def __call__(self, args):
+        super(FunctionalityUpdateCommand, self).__call__(args)
+        resource = None
+        for model in self.ls.funcs.list():
+            if model.get('identifier') == args.identifier:
+                resource = model
+                break
+        if resource is None:
+            raise ValueError("Functionality not found")
+        if self.debug:
+            self.pretty_json(resource)
+
+        if args.domain:
+            resource['domain'] = args.domain
+
+        policy = None
+        if args.activation_policy:
+            policy = resource.get('activationPolicy')
+        elif args.configuration_policy:
+            policy = resource.get('configurationPolicy')
+        elif args.delegation_policy:
+            policy = resource.get('delegationPolicy')
+        if policy:
+            if args.policy is not None:
+                policy['policy'] = args.policy
+            if args.status is not None:
+                policy['status'] = args.status
+
+        param = self.get_param(args, resource)
+        if param:
+            f_type = param.get('type')
+            if args.value is not None:
+                if  f_type == "INTEGER":
+                    param['integer'] = args.value
+                elif  f_type == "STRING":
+                    param['string'] = args.value
+                elif  f_type == "UNIT_SIZE":
+                    param['integer'] = args.value
+                elif  f_type == "UNIT_TIME":
+                    param['integer'] = args.value
+                elif  f_type == "BOOLEAN":
+                    param['bool'] = args.status
+            if args.unit is not None:
+                if  f_type == "UNIT_SIZE":
+                    param['string'] = args.unit
+                elif  f_type == "UNIT_TIME":
+                    param['string'] = args.unit
+
+        #self.pretty_json(resource)
+        #return True
+        return self._run(
+            self.ls.funcs.update,
+            "The following functinality '%(identifier)s' was successfully updated",
+            args.identifier,
+            resource)
+        #json_obj = self.ls.funcs.get(args.identifier, args.domain)
+        #keys = self.ls.funcs.get_rbu().get_keys()
+
+
+    def get_param(self, args, resource):
+        args.param = 1
+        pmax = len(resource.get('parameters'))
+        if args.param < 1 or args.param > pmax:
+            raise ValueError("invalid value for param.")
+        return resource['parameters'][args.param - 1]
+
+
+    def complete(self, args, prefix):
+        super(FunctionalityUpdateCommand, self).__call__(args)
+        json_obj = self.ls.funcs.list(args.domain)
+        return (v.get('identifier')
+                for v in json_obj if v.get('identifier').startswith(prefix))
+
+    def complete_domain(self, args, prefix):
+        super(FunctionalityUpdateCommand, self).__call__(args)
+        json_obj = self.ls.domains.list()
+        return (v.get('identifier')
+                for v in json_obj if v.get('identifier').startswith(prefix))
+
+    def complete_unit(self, args, prefix):
+        """ Complete with available units."""
+        super(FunctionalityUpdateCommand, self).__call__(args)
+        if args.identifier is None:
+            return ["error-identier-not-set",]
+        json_obj = self.ls.funcs.get(args.identifier, args.domain)
+        param = self.get_param(args, json_obj)
+        return param.get("select")
+
+    def complete_policies(self, args, prefix):
+        """ Complete with available policies."""
+        return "MANDATORY FORBIDDEN ALLOWED".split()
+
+
+# -----------------------------------------------------------------------------
+class FunctionalityResetCommand(DefaultCommand):
+    """ Reset a functionality."""
+
+    def __call__(self, args):
+        super(FunctionalityResetCommand, self).__call__(args)
+        json_obj = self.ls.funcs.get(args.identifier, args.domain)
+        if self.debug:
+            self.pretty_json(json_obj)
+        name = json_obj.get('identifier')
+        name += " (domain : " + json_obj.get('domain') + ")"
+        return self._delete(
+            self.ls.funcs.reset,
+            "The funtionality " + name + " was successfully reseted.",
+            name,
+            json_obj)
+
+    def complete(self, args, prefix):
+        super(FunctionalityResetCommand, self).__call__(args)
+        json_obj = self.ls.funcs.list(args.domain)
+        return (v.get('identifier')
+                for v in json_obj if v.get('identifier').startswith(prefix))
+
+    def complete_domain(self, args, prefix):
+        super(FunctionalityResetCommand, self).__call__(args)
+        json_obj = self.ls.domains.list()
+        return (v.get('identifier')
+                for v in json_obj if v.get('identifier').startswith(prefix))
+
+
+
 ###############################################################################
 ###  domains
 ###############################################################################
@@ -946,7 +1217,7 @@ def add_threads_parser(subparsers, name, desc):
 
 
 ###############################################################################
-###  threads
+###  thread members
 ###############################################################################
 def add_thread_members_parser(subparsers, name, desc):
     """Add all thread member sub commands."""
@@ -983,6 +1254,9 @@ def add_users_parser(subparsers, name, desc):
                                          help="list users from linshare")
     parser_tmp2.add_argument('identifiers', nargs="*",
                              help="").completer = Completer()
+    parser_tmp2.add_argument('-f', '--firstname', action="store")
+    parser_tmp2.add_argument('-l', '--lastname', action="store")
+    parser_tmp2.add_argument('-m', '--mail', action="store")
     parser_tmp2.add_argument('--extended', action="store_true",
                              help="extended format")
     parser_tmp2.add_argument('-r', '--reverse', action="store_true",
@@ -992,6 +1266,111 @@ def add_users_parser(subparsers, name, desc):
     parser_tmp2.set_defaults(__func__=UsersListCommand())
 
 
+###############################################################################
+###  functionalities
+###############################################################################
+def add_functionalitites_parser(subparsers, name, desc):
+    """Add all domain sub commands."""
+    parser_tmp = subparsers.add_parser(name, help=desc)
+    subparsers2 = parser_tmp.add_subparsers()
+
+    # command : list
+    parser_tmp2 = subparsers2.add_parser(
+        'list', help="list functionalities.")
+    parser_tmp2.add_argument('identifiers', nargs="*",
+                             help="").completer = Completer()
+    parser_tmp2.add_argument('-d', '--domain', action="store",
+                             help="").completer = Completer('complete_domain')
+    parser_tmp2.add_argument('--extended', action="store_true",
+                             help="extended format")
+    parser_tmp2.add_argument('-r', '--reverse', action="store_true",
+                             help="reverse order while sorting")
+    parser_tmp2.add_argument('-t', '--vertical', action="store_true",
+                             help="use vertical output mode")
+    parser_tmp2.set_defaults(__func__=FunctionalityListCommand())
+
+    # command : display
+    parser_tmp2 = subparsers2.add_parser(
+        'display', help="display a functionality.")
+    parser_tmp2.add_argument('identifier', action="store",
+                             help="").completer = Completer()
+    parser_tmp2.add_argument('-d', '--domain', action="store",
+                             help="").completer = Completer('complete_domain')
+    parser_tmp2.add_argument('--extended', action="store_true",
+                             help="extended format")
+    parser_tmp2.set_defaults(__func__=FunctionalityDisplayCommand())
+
+    # command : update
+    parser_tmp2 = subparsers2.add_parser(
+        'update', help="update a functionality.")
+    parser_tmp2.add_argument('identifier', action="store",
+                             help="").completer = Completer()
+    parser_tmp2.add_argument('-d', '--domain', action="store",
+                             help="Completion available").completer = Completer('complete_domain')
+        #return "".split()
+    policy_group = parser_tmp2.add_argument_group('Choose the policy to update')
+    group = policy_group.add_mutually_exclusive_group()
+    group.add_argument(
+        '-a',
+        '--activation-policy',
+        action="store_true",
+        help="activation policy")
+
+    group.add_argument(
+        '-c',
+        '--configuration-policy',
+        action="store_true",
+        help="configuration policy")
+
+    group.add_argument(
+        '-g',
+        '--delegation-policy',
+        action="store_true",
+        help="delegation policy")
+    parser_tmp2.add_argument(
+        '-p',
+        '--policy',
+        action="store",
+        help="MANDATORY, FORBIDDEN, ALLOWED"
+        ).completer = Completer('complete_policies')
+    status_group = parser_tmp2.add_argument_group('Status')
+    group = status_group.add_mutually_exclusive_group()
+    group.add_argument(
+        '--disable',
+        default=None,
+        action="store_false",
+        dest="status")
+    group.add_argument(
+        '--enable',
+        default=None,
+        action="store_true",
+        dest="status")
+
+    parser_tmp2.add_argument(
+        '-v',
+        '--value',
+        action="store")
+    parser_tmp2.add_argument(
+        '--param',
+        action="store",
+        default=1,
+        help=argparse.SUPPRESS)
+    parser_tmp2.add_argument(
+        '-u',
+        '--unit',
+        action="store").completer = Completer('complete_unit')
+
+    parser_tmp2.set_defaults(__func__=FunctionalityUpdateCommand())
+
+    # command : reset
+    parser_tmp2 = subparsers2.add_parser(
+        'reset', help="reset a functionality.")
+    parser_tmp2.add_argument('identifier', action="store",
+                             help="").completer = Completer()
+    parser_tmp2.add_argument('domain', action="store",
+                             help="").completer = Completer('complete_domain')
+    parser_tmp2.set_defaults(__func__=FunctionalityResetCommand())
+
 ###############################################################################
 ### test
 ###############################################################################
