@@ -37,12 +37,13 @@ import cookielib
 import json
 import poster
 import time
-#from datetime import datetime
 import datetime
 from ordereddict import OrderedDict
 from progressbar import ProgressBar, FileTransferSpeed, Bar, ETA, Percentage
 import hashlib
 
+# pylint: disable-msg=C0111
+# pylint: disable-msg=R0903
 # -----------------------------------------------------------------------------
 class LinShareException(Exception):
     pass
@@ -121,14 +122,14 @@ def cli_get_cache(user_function):
 
     def get_data(func, cachefile, *args):
         res = log_exec_time(func, *args)
-        with open(cachefile, 'wb') as f:
-            json.dump(res, f)
+        with open(cachefile, 'wb') as fde:
+            json.dump(res, fde)
         return res
 
     def _load_data(cachefile):
         if os.path.isfile(cachefile):
-            with open(cachefile, 'rb') as f:
-                res = json.load(f)
+            with open(cachefile, 'rb') as fde:
+                res = json.load(fde)
             return res
         else:
             raise ValueError("no file found.")
@@ -138,34 +139,30 @@ def cli_get_cache(user_function):
 
     def decorating_function(*args):
         cli = args[0]
-        url = cli.getFullUrl(args[1])
+        url = cli.get_full_url(args[1])
         cli.log.debug("cache url : " + url)
         key = hashlib.sha256(url + "|" + cli.user).hexdigest()
         cli.log.debug("key: " + key)
         cachefile = cachedir + "/" + key
-
         cache_time = cli.cache_time
         nocache = cli.nocache
-
         res = None
         if nocache:
             log.debug("cache disabled.")
             return log_exec_time(user_function, *args)
-
         if os.path.isfile(cachefile):
             file_time = os.stat(cachefile).st_mtime
-            a = "{da:%Y-%m-%d %H:%M:%S}"
+            form = "{da:%Y-%m-%d %H:%M:%S}"
             log.debug("cached data : " + str(
-                a.format(da=datetime.datetime.fromtimestamp(file_time))))
-
+                form.format(da=datetime.datetime.fromtimestamp(file_time))))
             if time.time() - cache_time > file_time:
                 log.debug("refreshing cached data.")
                 res = get_data(user_function, cachefile, *args)
             if not res:
                 try:
                     res = load_data(cachefile)
-                except ValueError as e:
-                    log.debug("error : " + str(e))
+                except ValueError as ex:
+                    log.debug("error : " + str(ex))
         if not res:
             res = get_data(user_function, cachefile, *args)
         return res
@@ -177,33 +174,26 @@ def cli_get_cache(user_function):
 class CoreCli(object):
 
     def __init__(self, host, user, password, verbose=False, debug=0,
-                 realm="Name Of Your LinShare Realm",
-                 application_name="linshare"):
+                 realm="Name Of Your LinShare Realm"):
         classname = str(self.__class__.__name__.lower())
         self.log = logging.getLogger('linshare-cli.' + classname)
         self.verbose = verbose
         self.debug = debug
         self.password = password
+        self.host = host
         self.user = user
         self.last_req_time = None
         self.cache_time = 60
         self.nocache = False
-
+        self.base_url = ""
         if not host:
             raise ValueError("invalid host : host url is not set !")
         if not user:
             raise ValueError("invalid user : " + str(user))
         if not password:
             raise ValueError("invalid password : password is not set ! ")
-
-        if not application_name:
-            application_name = "linshare"
         if not realm:
             realm = "Name Of Your LinShare Realm"
-
-        self.root_url = host + "/" + application_name
-        self.root_url += "/" + "webservice/rest/"
-
         self.debuglevel = 0
         # 0 : debug off
         # 1 : debug on
@@ -218,7 +208,6 @@ class CoreCli(object):
 
             if self.debuglevel >= 3:
                 httpdebuglevel = 1
-
         # We declare all the handlers useful here.
         auth_handler = urllib2.HTTPBasicAuthHandler()
         # we convert unicode objects to utf8 strings because
@@ -231,7 +220,6 @@ class CoreCli(object):
                 passwd=password.encode('utf8'))
         except UnicodeEncodeError:
             self.log.error("the program was not able to compute the basic authentication token.")
-
         handlers = [
             poster.streaminghttp.StreamingHTTPSHandler(
                 debuglevel=httpdebuglevel),
@@ -242,34 +230,38 @@ class CoreCli(object):
                 debuglevel=httpdebuglevel),
             poster.streaminghttp.StreamingHTTPRedirectHandler(),
             urllib2.HTTPCookieProcessor(cookielib.CookieJar())]
-
         # Setting handlers
         urllib2.install_opener(urllib2.build_opener(*handlers))
 
-    def getFullUrl(self, url_frament):
-        return self.root_url + url_frament
+    def get_full_url(self, url_frament):
+        root_url = self.host
+        if root_url[-1] != "/":
+            root_url += "/"
+        root_url += self.base_url
+        if root_url[-1] != "/":
+            root_url += "/"
+        root_url += url_frament
+        return root_url
 
     def auth(self):
-        url = self.getFullUrl("authentication/authorized")
+        url = self.get_full_url("authentication/authorized")
         self.log.debug("list url : " + url)
-
         # Building request
         request = urllib2.Request(url)
-
+        request.add_header('Content-Type', 'application/json; charset=UTF-8')
+        request.add_header('Accept', 'application/json')
         # doRequest
         try:
             resultq = urllib2.urlopen(request)
-
             code = resultq.getcode()
             if code == 200:
                 self.log.debug("auth url : ok")
                 return True
-
-        except urllib2.HTTPError as e:
-            if e.code == 401:
-                self.log.error(e.msg + " (" + str(e.code) + ")")
+        except urllib2.HTTPError as ex:
+            if ex.code == 401:
+                self.log.error(ex.msg + " (" + str(ex.code) + ")")
             else:
-                self.log.error(e.msg + " (" + str(e.code) + ")")
+                self.log.error(ex.msg + " (" + str(ex.code) + ")")
         return False
 
     def add_auth_header(self, request):
@@ -279,14 +271,14 @@ class CoreCli(object):
         #request.add_header("Cookie", "JSESSIONID=")
 
     def get_json_result(self, resultq):
-        jObj  = None
+        json_obj = None
         result = resultq.read()
         content_type = resultq.headers.getheader('Content-Type')
         if content_type == "application/json":
-            jObj = json.loads(result)
+            json_obj = json.loads(result)
             if self.debuglevel >= 2:
                 self.log.debug("the result is : ")
-                self.log.debug(json.dumps(jObj, sort_keys=True, indent=2))
+                self.log.debug(json.dumps(json_obj, sort_keys=True, indent=2))
         else:
             self.log.debug(str(result))
             msg = "Wrong content type in the http response : "
@@ -294,12 +286,12 @@ class CoreCli(object):
                 msg += content_type
             self.log.error(msg)
             raise ValueError(msg)
-        return jObj
+        return json_obj
 
-    def do_request(self, request):
+    def do_request(self, request, force_nocontent=False):
         # request start
         self.last_req_time = None
-        jObj = None
+        json_obj = None
         starttime = datetime.datetime.now()
         try:
             # doRequest
@@ -308,9 +300,12 @@ class CoreCli(object):
             if self.verbose or self.debug:
                 self.log.info("http return code : " + str(code))
             if code == 200:
-                jObj = self.get_json_result(resultq)
+                if force_nocontent:
+                    json_obj = resultq
+                else:
+                    json_obj = self.get_json_result(resultq)
             if code == 204:
-                jObj = True
+                json_obj = True
         except urllib2.HTTPError as ex:
             code = "-1"
             msg = "Http error : " + ex.msg + " (" + str(ex.code) + ")"
@@ -318,10 +313,10 @@ class CoreCli(object):
                 self.log.info(msg)
             else:
                 self.log.debug(msg)
-            if ex.code != 405:
-                jObj = self.get_json_result(ex)
-                code = jObj.get('errCode')
-                msg = jObj.get('message')
+            if ex.code == 400:
+                json_obj = self.get_json_result(ex)
+                code = json_obj.get('errCode')
+                msg = json_obj.get('message')
                 self.log.debug("Server error code : " + str(code))
                 self.log.debug("Server error message : " + str(msg))
             # request end
@@ -331,80 +326,100 @@ class CoreCli(object):
         # request end
         endtime = datetime.datetime.now()
         self.last_req_time = str(endtime - starttime)
-        return jObj
+        return json_obj
 
     @cli_get_cache
     def list(self, url):
         """ List ressources store into LinShare."""
-        url = self.root_url + url
+        url = self.get_full_url(url)
         self.log.debug("list url : " + url)
-
         # Building request
         request = urllib2.Request(url)
         request.add_header('Content-Type', 'application/json; charset=UTF-8')
         request.add_header('Accept', 'application/json')
-
+        # Do request
         ret = self.do_request(request)
-
         self.log.debug("""list url : %(url)s : request time : %(time)s""",
                        {"url": url,
                         "time": self.last_req_time})
         return ret
 
+    def get(self, url):
+        """ List ressources store into LinShare."""
+        url = self.get_full_url(url)
+        self.log.debug("get url : " + url)
+        # Building request
+        request = urllib2.Request(url)
+        request.add_header('Content-Type', 'application/json; charset=UTF-8')
+        request.add_header('Accept', 'application/json')
+        # Do request
+        ret = self.do_request(request)
+        self.log.debug("""get url : %(url)s : request time : %(time)s""",
+                       {"url": url,
+                        "time": self.last_req_time})
+        return ret
+
     def delete(self, url, data=None):
         """Delete one ressource store into LinShare."""
-        url = self.root_url + url
+        url = self.get_full_url(url)
         self.log.debug("delete url : " + url)
-
         # Building request
         request = urllib2.Request(url)
         if data:
             # Building request
             post_data = json.dumps(data).encode("UTF-8")
             request = urllib2.Request(url, post_data)
-            request.add_header('Content-Type', 'application/json; charset=UTF-8')
+            request.add_header('Content-Type',
+                               'application/json; charset=UTF-8')
             request.add_header('Accept', 'application/json')
-
         request.get_method = lambda: 'DELETE'
-
         ret = self.do_request(request)
-
         self.log.debug("""delete url : %(url)s : request time : %(time)s""",
                        {"url": url,
                         "time": self.last_req_time})
         return ret
 
     def options(self, url):
-        url = self.root_url + url
+        url = self.get_full_url(url)
         self.log.debug("options url : " + url)
-
         # Building request
         request = urllib2.Request(url)
         request.add_header('Content-Type', 'application/json; charset=UTF-8')
         request.add_header('Accept', 'application/json')
-
         request.get_method = lambda: 'OPTIONS'
-
+        # Do request
         ret = self.do_request(request)
-
         self.log.debug("""options url : %(url)s : request time : %(time)s""",
                        {"url": url,
                         "time": self.last_req_time})
         return ret
 
+    def head(self, url, **kwargs):
+        url = self.get_full_url(url)
+        self.log.debug("head url : " + url)
+        # Building request
+        request = urllib2.Request(url)
+        request.add_header('Content-Type', 'application/json; charset=UTF-8')
+        request.add_header('Accept', 'application/json')
+        request.get_method = lambda: 'HEAD'
+        # Do request
+        ret = self.do_request(request, **kwargs)
+        self.log.debug("""head url : %(url)s : request time : %(time)s""",
+                       {"url": url,
+                        "time": self.last_req_time})
+        return ret
+
     def create(self, url, data):
         """ create ressources store into LinShare."""
-        url = self.root_url + url
+        url = self.get_full_url(url)
         self.log.debug("create url : " + url)
-
         # Building request
         post_data = json.dumps(data).encode("UTF-8")
         request = urllib2.Request(url, post_data)
         request.add_header('Content-Type', 'application/json; charset=UTF-8')
         request.add_header('Accept', 'application/json')
-
+        # Do request
         ret = self.do_request(request)
-
         self.log.debug("""post url : %(url)s : request time : %(time)s""",
                        {"url": url,
                         "time": self.last_req_time})
@@ -412,18 +427,16 @@ class CoreCli(object):
 
     def update(self, url, data):
         """ update ressources store into LinShare."""
-        url = self.root_url + url
+        url = self.get_full_url(url)
         self.log.debug("update url : " + url)
-
         # Building request
         post_data = json.dumps(data).encode("UTF-8")
         request = urllib2.Request(url, post_data)
         request.add_header('Content-Type', 'application/json; charset=UTF-8')
         request.add_header('Accept', 'application/json')
         request.get_method = lambda: 'PUT'
-
+        # Do request
         ret = self.do_request(request)
-
         self.log.debug("""put url : %(url)s : request time : %(time)s""",
                        {"url": url,
                         "time": self.last_req_time})
@@ -431,72 +444,57 @@ class CoreCli(object):
 
     def upload(self, file_path, url, description=None):
         self.last_req_time = None
-        url = self.root_url + url
+        url = self.get_full_url(url)
         self.log.debug("upload url : " + url)
-
         # Generating datas and headers
         file_size = os.path.getsize(file_path)
-
         self.log.debug("file_path is : " + file_path)
         file_name = os.path.basename(file_path)
         self.log.debug("file_name is : " + file_name)
-
         if file_size <= 0:
             msg = "The file '%(filename)s' can not be uploaded \
 because its size is less or equal to zero." % {"filename": str(file_name)}
             raise LinShareException("-1", msg)
-
         widgets = [FileTransferSpeed(), ' <<<', Bar(), '>>> ',
                    Percentage(), ' ', ETA()]
         pbar = ProgressBar(widgets=widgets, maxval=file_size)
         stream = FileWithCallback(file_path, 'rb', pbar.update,
-                                    file_size, file_path)
-
-
-        p = poster.encode.MultipartParam("file", filename=file_name,
-                                         fileobj=stream)
-        params = [p,]
+                                  file_size, file_path)
+        post = poster.encode.MultipartParam("file", filename=file_name,
+                                            fileobj=stream)
+        params = [post,]
         if description:
             params.append(("description", description))
         datagen, headers = poster.encode.multipart_encode(params)
-
         # Building request
         request = urllib2.Request(url, datagen, headers)
         request.add_header('Accept', 'application/json')
-
         # request start
         pbar.start()
         starttime = datetime.datetime.now()
-
         resultq = None
         try:
             # doRequest
             resultq = urllib2.urlopen(request)
             code = resultq.getcode()
-
             if self.verbose or self.debug:
                 self.log.info("http return code : " + str(code))
-
             if code == 200:
-                jObj = self.get_json_result(resultq)
-
+                json_obj = self.get_json_result(resultq)
         except urllib2.HTTPError as ex:
-
             if self.verbose:
                 self.log.info("Http error : " + ex.msg + " (" + str(ex.code) + ")")
             else:
                 self.log.debug("Http error : " + ex.msg + " (" + str(ex.code) + ")")
-            jObj = self.get_json_result(ex)
-            code = jObj.get('errCode')
-            msg = jObj.get('message')
+            json_obj = self.get_json_result(ex)
+            code = json_obj.get('errCode')
+            msg = json_obj.get('message')
             self.log.debug("Server error code : " + str(code))
             self.log.debug("Server error message : " + str(msg))
-
             # request end
             endtime = datetime.datetime.now()
             pbar.finish()
             self.last_req_time = str(endtime - starttime)
-
             if ex.code == 502:
                 self.log.warn(
                     """The file '%(filename)s' was uploaded
@@ -510,23 +508,20 @@ because its size is less or equal to zero." % {"filename": str(file_name)}
                     {"filename": file_name,
                      "filepath": file_path})
             raise LinShareException(code, msg)
-
-
         # request end
         endtime = datetime.datetime.now()
         pbar.finish()
-
         self.last_req_time = str(endtime - starttime)
         self.log.debug("upload url : %(url)s : request time : %(time)s",
                        {"url": url,
                         "time": self.last_req_time})
-        return jObj
+        return json_obj
 
     def download(self, uuid, url):
         """ download a file from LinShare using its rest api.
 This method could throw exceptions like urllib2.HTTPError."""
         self.last_req_time = None
-        url = self.root_url + url
+        url = self.get_full_url(url)
         self.log.debug("download url : " + url)
 
         # Building request
@@ -566,7 +561,7 @@ This method could throw exceptions like urllib2.HTTPError."""
                        Percentage(), ' ', ETA()]
             pbar = ProgressBar(widgets=widgets, maxval=file_size)
             stream = FileWithCallback(file_name, 'w', pbar.update,
-                                        file_size, file_name)
+                                      file_size, file_name)
             pbar.start()
 
             chunk_size = 8192
@@ -735,30 +730,24 @@ class Shares(object):
         self.log = logging.getLogger('linshare-cli.' + classname)
 
     def share(self, uuid, mail):
-
-        url = self.core.root_url + "shares/sharedocument/%s/%s" % (mail, uuid)
+        url = self.core.get_full_url("shares/sharedocument/%s/%s" % (mail, uuid))
         self.log.debug("share url : " + url)
-
         # Building request
         request = urllib2.Request(url)
-
         # request start
         starttime = datetime.datetime.now()
-
         try:
             # doRequest
-                resultq = urllib2.urlopen(request)
-        except urllib2.HTTPError as e:
-                print e
-                print e.code
-                print url
-                raise e
-
+            resultq = urllib2.urlopen(request)
+        except urllib2.HTTPError as ex:
+            print ex
+            print ex.code
+            print url
+            raise ex
         # request end
         endtime = datetime.datetime.now()
         code = resultq.getcode()
         msg = resultq.msg
-
         self.core.last_req_time = str(endtime - starttime)
         self.log.debug("share url : %(url)s : request time : %(time)s",
                        {"url": url,
@@ -781,8 +770,8 @@ class ThreadsMembers(object):
     def __init__(self, corecli):
         self.core = corecli
 
-    def list(self, threadUuid):
-        url = "thread_members/%s" % threadUuid
+    def list(self, thread_uuid):
+        url = "thread_members/%s" % thread_uuid
         return self.core.list(url)
 
 
@@ -799,6 +788,7 @@ class Users(object):
 class UserCli(CoreCli):
     def __init__(self, *args, **kwargs):
         super(UserCli, self).__init__(*args, **kwargs)
+        self.base_url = "linshare/webservice/rest"
         self.documents = Documents(self)
         self.rshares = ReceivedShares(self)
         self.shares = Shares(self)
@@ -826,7 +816,7 @@ class GenericAdminClass(object):
         self.log.debug("input data :")
         self.log.debug(json.dumps(data, sort_keys=True, indent=2))
 
-    def check(self, data):
+    def _check(self, data):
         rbu = self.get_rbu()
         rbu.copy(data)
         rbu.check_required_fields()
@@ -835,21 +825,21 @@ class GenericAdminClass(object):
 class DomainAdmins(GenericAdminClass):
 
     def list(self):
-        return self.core.list("admin/domains")
+        return self.core.list("domains")
 
     def create(self, data):
         self.debug(data)
         if data.get('label') is None:
             data['label'] = data.get('identifier')
-        self.check(data)
+        self._check(data)
         if data.get('type') in ["GUESTDOMAIN", "SUBDOMAIN"]:
             if data.get('parent') is None:
                 raise ValueError("parent identifier is required for GuestDomain or SubDomain")
-        return self.core.create("admin/domains", data)
+        return self.core.create("domains", data)
 
     def update(self, data):
         self.debug(data)
-        return self.core.update("admin/domains", data)
+        return self.core.update("domains", data)
 
     def delete(self, identifier):
         if identifier:
@@ -857,17 +847,17 @@ class DomainAdmins(GenericAdminClass):
         if not identifier:
             raise ValueError("identifier is required")
         data = {"identifier":  identifier}
-        return self.core.delete("admin/domains", data)
+        return self.core.delete("domains", data)
 
     def options_language(self):
-        return self.core.options("admin/enums/language")
+        return self.core.options("enums/language")
 
     def options_role(self):
-        my_list = self.core.options("admin/enums/role")
+        my_list = self.core.options("enums/role")
         return filter(lambda x: x not in ["SUPERADMIN", "SYSTEM"], my_list)
 
     def options_type(self):
-        my_list = self.core.options("admin/enums/domain_type")
+        my_list = self.core.options("enums/domain_type")
         return filter(lambda x: x != "ROOTDOMAIN", my_list)
 
 
@@ -892,18 +882,18 @@ class DomainPatternsAdmin(GenericAdminClass):
 
     def list(self, model=False):
         if model:
-            return self.core.list("admin/domain_patterns/models")
+            return self.core.list("domain_patterns/models")
         else:
-            return self.core.list("admin/domain_patterns")
+            return self.core.list("domain_patterns")
 
     def create(self, data):
         self.debug(data)
-        self.check(data)
-        return self.core.create("admin/domain_patterns", data)
+        self._check(data)
+        return self.core.create("domain_patterns", data)
 
     def update(self, data):
         self.debug(data)
-        return self.core.update("admin/domain_patterns", data)
+        return self.core.update("domain_patterns", data)
 
     def delete(self, identifier):
         if identifier:
@@ -911,7 +901,7 @@ class DomainPatternsAdmin(GenericAdminClass):
         if not identifier:
             raise ValueError("identifier is required")
         data = {"identifier":  identifier}
-        return self.core.delete("admin/domain_patterns", data)
+        return self.core.delete("domain_patterns", data)
 
     def get_rbu(self):
         rbu = ResourceBuilder("domain_patterns", required=True)
@@ -935,16 +925,16 @@ class DomainPatternsAdmin(GenericAdminClass):
 class LdapConnectionsAdmin(GenericAdminClass):
 
     def list(self):
-        return self.core.list("admin/ldap_connections")
+        return self.core.list("ldap_connections")
 
     def create(self, data):
         self.debug(data)
-        self.check(data)
-        return self.core.create("admin/ldap_connections", data)
+        self._check(data)
+        return self.core.create("ldap_connections", data)
 
     def update(self, data):
         self.debug(data)
-        return self.core.update("admin/ldap_connections", data)
+        return self.core.update("ldap_connections", data)
 
     def delete(self, identifier):
         if identifier:
@@ -952,7 +942,7 @@ class LdapConnectionsAdmin(GenericAdminClass):
         if not identifier:
             raise ValueError("identifier is required")
         data = {"identifier":  identifier}
-        return self.core.delete("admin/ldap_connections", data)
+        return self.core.delete("ldap_connections", data)
 
     def get_rbu(self):
         rbu = ResourceBuilder("ldap_connection")
@@ -967,7 +957,7 @@ class LdapConnectionsAdmin(GenericAdminClass):
 class ThreadsAdmin(GenericAdminClass):
 
     def list(self):
-        return self.core.list("admin/threads")
+        return self.core.list("threads")
 
     def get_rbu(self):
         rbu = ResourceBuilder("threads")
@@ -981,8 +971,8 @@ class ThreadsAdmin(GenericAdminClass):
 # -----------------------------------------------------------------------------
 class ThreadsMembersAdmin(GenericAdminClass):
 
-    def list(self, threadUuid):
-        url = "admin/thread_members/%s" % threadUuid
+    def list(self, thread_uuid):
+        url = "thread_members/%s" % thread_uuid
         return self.core.list(url)
 
     def get_rbu(self):
@@ -993,38 +983,99 @@ class ThreadsMembersAdmin(GenericAdminClass):
 # -----------------------------------------------------------------------------
 class UsersAdmin(GenericAdminClass):
 
-    def list(self):
-        return self.core.list("users")
+    def search(self, firstname=None, lastname=None, mail=None):
+        criteria = {"firstName": firstname,
+                    "lastName": lastname,
+                    "mail": mail}
+        return self.core.create("users/search", criteria)
+
+    def autocomplete(self, pattern):
+        if not pattern:
+            raise ValueError("missing mandatory parameter : pattern")
+        return self.core.list("users/autocomplete/%s" % pattern)
+
+    def internals(self, pattern):
+        if not pattern:
+            raise ValueError("missing mandatory parameter : pattern")
+        return self.core.list("users/search/internals/%s" % pattern)
+
+    def guests(self, pattern):
+        if not pattern:
+            raise ValueError("missing mandatory parameter : pattern")
+        return self.core.list("users/search/guests/%s" % pattern)
+
+    def inconsistents(self):
+        return self.core.list("users/inconsistent")
 
     def get_rbu(self):
-        #"canCreateGuest": null,
-        #"canUpload": null,
-        #"comment": null,
-        #"expirationDate": null,
-        #"guest": false,
-        #"owner": null,
-        #"restricted": null,
-        #"restrictedContacts": [],
         rbu = ResourceBuilder("users")
         rbu.add_field('firstName', required=True)
         rbu.add_field('lastName', required=True)
         rbu.add_field('mail', required=True)
         rbu.add_field('uuid')
         rbu.add_field('domain')
+        rbu.add_field('guest')
         rbu.add_field('role')
         rbu.add_field('locale')
         rbu.add_field('creationDate')
         rbu.add_field('modificationDate')
+        rbu.add_field('canUpload', extended=True)
+        rbu.add_field('canCreateGuest', extended=True)
+        rbu.add_field('restricted', extended=True)
+        rbu.add_field('expirationDate', extended=True)
+        rbu.add_field('comment', extended=True)
+        #rbu.add_field('owner', extended=True)
+        rbu.add_field('restrictedContacts', extended=True)
+        return rbu
+
+# -----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+class FunctionalityAdmin(GenericAdminClass):
+
+    def list(self, domain_id=None):
+        if domain_id is None:
+            domain_id = "LinShareRootDomain"
+        json_obj = self.core.list("functionalities?domainId=" + domain_id)
+        return filter(lambda x: x.get('displayable') == True, json_obj)
+
+    def get(self, func_id, domain_id=None):
+        if domain_id is None:
+            domain_id = "LinShareRootDomain"
+        json_obj = self.core.get("functionalities/"+ func_id +"?domainId=" +
+                                 domain_id)
+        return json_obj
+
+    def update(self, data):
+        self.debug(data)
+        return self.core.update("functionalities", data)
+
+    def reset(self, data):
+        self.debug(data)
+        return self.core.delete("functionalities", data)
+
+    def options_policies(self):
+        return self.core.options("enums/policies")
+
+    def get_rbu(self):
+        rbu = ResourceBuilder("functionality")
+        #rbu.add_field('functionalities')
+        rbu.add_field('identifier', required=True)
+        rbu.add_field('type')
+        rbu.add_field('parentAllowParametersUpdate')
+        rbu.add_field('parameters', extended=True)
+        rbu.add_field('parentIdentifier', extended=True)
+        rbu.add_field('domain', extended=True, required=True)
         return rbu
 
 # -----------------------------------------------------------------------------
 class AdminCli(CoreCli):
     def __init__(self, *args, **kwargs):
         super(AdminCli, self).__init__(*args, **kwargs)
+        self.base_url = "linshare/webservice/rest/admin"
         self.threads = ThreadsAdmin(self)
         self.thread_members = ThreadsMembersAdmin(self)
         self.users = UsersAdmin(self)
         self.domains = DomainAdmins(self)
         self.ldap_connections = LdapConnectionsAdmin(self)
         self.domain_patterns = DomainPatternsAdmin(self)
-
+        self.funcs = FunctionalityAdmin(self)
